diff --git a/src/backend/optimizer/plan/planner.c b/src/backend/optimizer/plan/planner.c
index a3cc274..3252904 100644
--- a/src/backend/optimizer/plan/planner.c
+++ b/src/backend/optimizer/plan/planner.c
@@ -1985,6 +1985,22 @@ grouping_planner(PlannerInfo *root, double tuple_fraction)
 					 * the desired tlist.
 					 */
 					result_plan->targetlist = sub_tlist;
+
+					/*
+					 * If the plan happens to be a Gather plan, and the
+					 * subplan is projection-capable, then insert the tlist
+					 * there, too, so that the workers can help with
+					 * projection.  But if there is something that is not
+					 * parallel-safe in the tlist, then we can't.
+					 */
+					if (IsA(result_plan, Gather))
+					{
+						Plan	   *outer_plan = outerPlan(result_plan);
+
+						if (is_projection_capable_plan(outer_plan) &&
+							!has_parallel_hazard((Node *) sub_tlist, false))
+							outer_plan->targetlist = sub_tlist;
+					}
 				}
 
 				/*
